/* An implementation of a non-preemptive, work-conserving
 * periodic EDF scheduler. */

#include "Arduino.h"

#include "tasks.h"
#include "sched_time.h"

static unsigned long next_release_time[MAX_TASKS];
static unsigned long absolute_deadline[MAX_TASKS];
static bool pending[MAX_TASKS];

void edf_init(void)
{
	for (int i = 0; i < MAX_TASKS; i++) {
		next_release_time[i] = 0UL;
		pending[i] = false;
		absolute_deadline[i] = 0UL;
	}
}

static void edf_release_job(byte i)
{
	/* unsigned long: possible overflow accepted and ok */
	absolute_deadline[i]  = next_release_time[i] + tasks[i].deadline;
	next_release_time[i] += tasks[i].period;
	pending[i] = true;
}

sched_decision_t edf_schedule(void)
{
	unsigned long now = current_sched_time();
	char earliest = -1;

	for (byte i = 0; i < number_of_tasks; i++)
	{
		if (!pending[i] && time_after(now, next_release_time[i])) {
			/* a job of this task has been released */
			edf_release_job(i);
		}
		if (pending[i] &&
		    (earliest < 0 || time_after(absolute_deadline[earliest],
		                                absolute_deadline[i]))) {
			/* consider this job for scheduling */
			earliest = i;
		}
	}

	if (earliest < 0) {
		/* No job is ready. Just idle. */
		return NO_JOB_READY;
	} else {
		/* schedule the job with the earliest deadline */
		pending[earliest] = false;
		return (sched_decision_t)
			{tasks[earliest].job_function, tasks[earliest].wcet};
	}
}
