#!/usr/bin/env python3

import sys
import os
import math

from hyperperiod import compute_hyperperiod

MAX_TASKS = 16

f = open("gentab_oe.h", "w")

print("""
/* NOTE: This file is automatically generated and should be included
 * only from sched_oe.cpp. */

static PROGMEM const struct idle_time_record idle_times[] = {""", file=f)

idle = 0
for line in open(sys.argv[1] + '/it.txt').readlines():
    slot = [int(x) for x in line.strip().split(" ")]
    print("\t{{ {0:>7d}, {1:>7d}UL }},".format(slot[0], slot[1]),
        file=f)
    idle += 1

# if not idle:
#     # insert dummy entry
#     print("\t{{ {0:>7d}, {1:>7d}UL }},".format(0, 0), file=f)

print("};", file=f)


print("""
#define idle_times_len {0}U
""".format(idle), file=f)

print("""
static PROGMEM const struct irregularity_record all_irregs[] = {""", file=f)

irregs_per_task = []
index_per_task = []
last_task = -1
index = 0
for line in open(sys.argv[1] + '/pi.txt').readlines():
    slot = [int(x) for x in line.strip().split(" ")]

    while last_task != slot[0]:
        if last_task >= 0:
            irregs_per_task.append(irregs)
        last_task += 1
        irregs = 0
        if last_task < MAX_TASKS:
            index_per_task.append(index)
            print("\t/* Task {0} */".format(last_task), file=f)

    print("\t{{ {0:>4d}, {1:>7d}UL }},".format(slot[1], slot[2]),
        file=f)
    irregs += 1
    index  += 1

while last_task != MAX_TASKS:
    if last_task >= 0:
        irregs_per_task.append(irregs)
    last_task += 1
    irregs = 0
    if last_task < MAX_TASKS:
        index_per_task.append(index)
        print("\t/* Task {0} */".format(last_task), file=f)

# if not irregs:
#     # insert dummy entry
#     print("\t{{ {0:>7d}, {1:>7d}UL }},".format(0, 0), file=f)

print("};", file=f)

print("""
static PROGMEM const struct irregularity_record* const irregs[MAX_TASKS] = {""", file=f)

for n in index_per_task:
    print("\t&all_irregs[{0:d}],".format(n), file=f)

print("};", file=f)

print("""
static PROGMEM const unsigned int irregs_len[MAX_TASKS] = {""", file=f)

for n in irregs_per_task:
    print("\t{0:>5d},".format(n), file=f)

print("};", file=f)

print("""
#define hyperperiod {0}UL
""".format(compute_hyperperiod(sys.argv[1] + '/ts.txt')), file=f)

f.close()


