#!/usr/bin/env python3


import sys
import os
import csv

# columns
SCHED = 0
NUM_TASKS = 1
EXP_ID = 2
INVOCATIONS = 3
TOTAL_OHEAD = 4
MAX_OHEAD = 5
MIN_OHEAD = 6

def summarize(fname):
    data = csv.reader(open(fname), delimiter=',')

    scheds = set()
    num_tasks = set()
    total_samples = 0
    total_cost    = 0
    min_cost      = None
    max_cost      = None

    for row in data:
        scheds.add(row[SCHED].strip())
        num_tasks.add(row[NUM_TASKS].strip())
        total_samples += int(row[INVOCATIONS])
        total_cost    += int(row[TOTAL_OHEAD])
        _min = int(row[MIN_OHEAD])
        _max = int(row[MAX_OHEAD])
        if min_cost is None or min_cost > _min:
            min_cost = _min
        if max_cost is None or max_cost < _max:
            max_cost = _max

    print("{0:>2s}, {1:>2s}, {2:7d}, {3:10d}, {4:7d}, {5:9.2f}, {6:7d}".format(
        " ".join(scheds),
        " ".join(num_tasks),
        total_samples,
        total_cost,
        max_cost,
        total_cost / total_samples,
        min_cost
    ))

if __name__ == '__main__':
    for path in sys.argv[1:]:
        summarize(path)


